-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 4515 of the CMF schema.
--
-- Adds role config group history support.
--

update SCHEMA_VERSION set VERSION=4516;

-- Add a FK to the config revision for role config groups and populate it
alter table ROLE_CONFIG_GROUPS 
  add column REVISION_ID bigint,
  add index IDX_ROLE_CONFIG_GROUP_CONFIG_REVISION (REVISION_ID),
  add constraint FK_ROLE_CONFIG_GROUP_CONFIG_REVISION
  foreign key (REVISION_ID)
  references CONFIG_REVISIONS (REVISION_ID);

update ROLE_CONFIG_GROUPS
  set REVISION_ID=(
    select MIN(REVISION_ID) from CONFIG_REVISIONS cr 
    where ROLE_CONFIG_GROUPS.SERVICE_ID=cr.SERVICE_ID);

alter table ROLE_CONFIG_GROUPS
  modify column REVISION_ID bigint not null;

-- Create the config revision <-> RCG many-to-many association table
create table CONF_REV_TO_ROLE_CONF_GRP (                   
  REVISION_ID bigint not null,
  ROLE_CONFIG_GROUP_ID bigint not null,
  primary key (REVISION_ID, ROLE_CONFIG_GROUP_ID)
) engine=InnoDB;                    
                                   
alter table CONF_REV_TO_ROLE_CONF_GRP
  add index IDX_CONF_REV_TO_ROLE_CONF_GRP_REVISION (REVISION_ID),
  add constraint FK_REV_TO_ROLE_CONF_GRP_REVISION
  foreign key (REVISION_ID)
  references CONFIG_REVISIONS (REVISION_ID);

alter table CONF_REV_TO_ROLE_CONF_GRP
  add index IDX_CONF_REV_TO_ROLE_CONF_GRP_RCG (ROLE_CONFIG_GROUP_ID),
  add constraint FK_REV_TO_ROLE_CONF_GRP_RCG
  foreign key (ROLE_CONFIG_GROUP_ID)
  references ROLE_CONFIG_GROUPS (ROLE_CONFIG_GROUP_ID);


-- Populate the config revision <-> RCG many-to-many association table
-- Associate every RCG with every revision in its service
insert into CONF_REV_TO_ROLE_CONF_GRP (REVISION_ID, ROLE_CONFIG_GROUP_ID) 
  (select cr.REVISION_ID, rcg.ROLE_CONFIG_GROUP_ID 
  from CONFIG_REVISIONS cr left join ROLE_CONFIG_GROUPS rcg 
  on cr.SERVICE_ID=rcg.SERVICE_ID 
  where cr.SERVICE_ID is not null);


-- Create the RCGs <-> Roles many-to-many association table
create table ROLE_CONF_GRP_TO_ROLE (                   
  ROLE_CONFIG_GROUP_ID bigint not null,
  ROLE_ID bigint not null,
  primary key (ROLE_CONFIG_GROUP_ID, ROLE_ID)
) engine=InnoDB;                    
                                   
alter table ROLE_CONF_GRP_TO_ROLE
  add index IDX_ROLE_CONF_GRP_TO_ROLE_RCG (ROLE_CONFIG_GROUP_ID),
  add constraint FK_ROLE_CONF_GRP_TO_ROLE_RCG
  foreign key (ROLE_CONFIG_GROUP_ID)
  references ROLE_CONFIG_GROUPS (ROLE_CONFIG_GROUP_ID);

alter table ROLE_CONF_GRP_TO_ROLE
  add index IDX_ROLE_CONF_GRP_TO_ROLE_ROLE (ROLE_ID),
  add constraint FK_ROLE_CONF_GRP_TO_ROLE_ROLE
  foreign key (ROLE_ID)
  references ROLES (ROLE_ID);


-- Populate the RCGs <-> Roles many-to-many association table
-- Associate every role with the current RCG it belongs to
insert into ROLE_CONF_GRP_TO_ROLE (ROLE_CONFIG_GROUP_ID, ROLE_ID) 
  (select r.ROLE_CONFIG_GROUP_ID, r.ROLE_ID
  from ROLES r);


-- Get rid of the service id column and associated FK constraint
alter table ROLE_CONFIG_GROUPS
  drop foreign key FK_ROLE_CONFIG_GROUP_SERVICE;

alter table ROLE_CONFIG_GROUPS 
  drop index SERVICE_ID;

alter table ROLE_CONFIG_GROUPS
  add unique index (REVISION_ID, ROLE_TYPE, NAME);

alter table ROLE_CONFIG_GROUPS
  drop column SERVICE_ID;
